/*
 *  Kylin national secret algorithm module
 *
 *  This file contains the kdf2 data structures
 *  and functions declared.
 *
 *  Authors:  YangZhao <yangzhao@kylinos.cn>
 *
 *  Copyright (C) 2017,Tianjin KYLIN Information Technology Co., Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the
 *  Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#ifndef HEADER_KDF2_H
#define HEADER_KDF2_H

#include <openssl/evp.h>
#include "gm/kdf.h"

#ifdef __cplusplus
extern "C" {
#endif


typedef void *(*KDF_FUNC)(const void *in, size_t inlen, void *out, size_t *outlen);

KDF_FUNC KDF_get_x9_63(const EVP_MD *md);
KDF_FUNC KDF_get_ibcs(const EVP_MD *md);
/*
KDF_FUNC KDF_get_nist_concatenation(void);
KDF_FUNC KDF_get_tls_kdf(void);
KDF_FUNC KDF_get_ikev2_kdf(void);
*/

/* BEGIN ERROR CODES */
/*
 * The following lines are auto generated by the script mkerr.pl. Any changes
 * made after this point may be overwritten when the script is next run.
 */

int ERR_load_KDF2_strings(void);

/* Error codes for the KDF2 functions. */

/* Function codes. */
# define KDF2_F_IBCS_KDF                                  100
# define KDF2_F_X963_KDF                                  101

/* Reason codes. */
# define KDF2_R_DIGEST_FAILURE                            100

# ifdef  __cplusplus
}
# endif
#endif
